package function.builtin;

import java.io.*;
import java.text.MessageFormat;

import environment.RuntimeEnvironment;
import error.*;
import function.*;
import parser.LispParser;
import sexpression.*;

public class LOAD extends LispFunction {

    private ArgumentValidator argumentValidator;
    private RuntimeEnvironment environment;
    private ErrorManager errorManager;

    public LOAD() {
        this.argumentValidator = new ArgumentValidator("LOAD");
        this.argumentValidator.setExactNumberOfArguments(1);
        this.argumentValidator.setEveryArgumentExpectedType(LispString.class);
        this.environment = RuntimeEnvironment.getInstance();
        this.errorManager = this.environment.getErrorManager();
    }

    public SExpression call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        LispString quotedName = (LispString) argumentList.getCar();
        String fileName = removeSurroundingQuotes(quotedName.toString());

        return processFile(fileName);
    }

    private String removeSurroundingQuotes(String fileName) {
        return fileName.substring(1, (fileName.length() - 1));
    }

    private SExpression processFile(String fileName) {
        boolean wasSuccessful = false;
        LispParser parser = attemptToCreateParserOnFile(fileName);

        if (parser != null)
            wasSuccessful = isSuccessfulEvaluation(parser);

        return wasSuccessful ? Symbol.T : Nil.getInstance();
    }

    private LispParser attemptToCreateParserOnFile(String fileName) {
        LispParser parser = null;

        try {
            parser = new LispParser(new FileInputStream(fileName), fileName);
        } catch (FileNotFoundException e) {
            errorManager.generateError(new CouldNotLoadFileException(fileName));
        }

        return parser;
    }

    private boolean isSuccessfulEvaluation(LispParser parser) {
        while (!parser.isEof()) {
            try {
                EVAL.eval(parser.getNextSExpression());
            } catch (LispException e) {
                errorManager.generateError(e);
                return false;
            }
        }

        return true;
    }

    public static class CouldNotLoadFileException extends LispException {

        private static final long serialVersionUID = 1L;
        private String fileName;

        public CouldNotLoadFileException(String fileName) {
            this.fileName = fileName;
        }

        @Override
        public String getMessage() {
            return MessageFormat.format("could not load ''{0}''", fileName);
        }
    }

}
