package interpreter;

import java.io.*;

import environment.RuntimeEnvironment;
import error.*;

public class LispInterpreterBuilderImpl implements LispInterpreterBuilder {

    private static LispInterpreterBuilder uniqueInstance = new LispInterpreterBuilderImpl();

    public static LispInterpreterBuilder getInstance() {
        return uniqueInstance;
    }

    private String inputName;
    private InputStream inputStream;
    private PrintStream outputStream;
    private PrintStream errorOutputStream;
    private Runnable terminationFunction;
    private Runnable errorTerminationFunction;
    private RuntimeEnvironment environment;
    private boolean isInteractive;
    private boolean isBuilt;

    private LispInterpreterBuilderImpl() {
        this.environment = RuntimeEnvironment.getInstance();
        this.inputName = "";
        this.isInteractive = true;
        this.isBuilt = false;
    }

    @Override
    public void setInputName(String inputName) {
        this.inputName = inputName;
    }

    @Override
    public void setInput(InputStream inputStream) {
        this.inputStream = inputStream;
    }

    @Override
    public void setOutput(PrintStream outputStream) {
        this.outputStream = outputStream;

    }

    @Override
    public void setErrorOutput(PrintStream errorOutputStream) {
        this.errorOutputStream = errorOutputStream;
    }

    @Override
    public void setTerminationFunction(Runnable terminationFunction) {
        this.terminationFunction = terminationFunction;
    }

    @Override
    public void setErrorTerminationFunction(Runnable errorTerminationFunction) {
        this.errorTerminationFunction = errorTerminationFunction;
    }

    @Override
    public void useFile(String fileName) {
        this.inputName = fileName;
        this.isInteractive = false;
    }

    @Override
    public LispInterpreter build() {
        if (!isBuilt)
            return buildInterpreter();
        else
            throw new InterpreterAlreadyBuiltException();
    }

    private LispInterpreter buildInterpreter() {
        configureRuntimeEnvironment();
        LispInterpreter lispInterpreter = createInterpreter();
        isBuilt = true;

        return lispInterpreter;
    }

    private void configureRuntimeEnvironment() {
        ErrorManager errorManager = new ErrorManager();

        environment.setOutput(outputStream);
        environment.setErrorOutput(errorOutputStream);
        environment.setTerminationFunction(terminationFunction);
        environment.setErrorTerminationFunction(errorTerminationFunction);
        environment.setErrorManager(errorManager);
        configureInput(errorManager);
    }

    private void configureInput(ErrorManager errorManager) {
        environment.setInputName(inputName);

        try {
            environment.setInput(getInputStream());
        } catch (FileNotFoundException e) {
            errorManager.generateError(new LispFileNotFoundException(e));
        }
    }

    private InputStream getInputStream() throws FileNotFoundException {
        return isInteractive ? inputStream : new FileInputStream(inputName);
    }

    private LispInterpreter createInterpreter() {
        return isInteractive ? new InteractiveLispInterpreter() : new LispInterpreter();
    }

    public class InterpreterAlreadyBuiltException extends LispException {

        private static final long serialVersionUID = 1L;

        @Override
        public int getSeverity() {
            return ErrorManager.CRITICAL_LEVEL;
        }

        @Override
        public String getMessage() {
            return "Refusing to build more than one interpreter.";
        }
    }

    public static class LispFileNotFoundException extends LispException {

        private static final long serialVersionUID = 1L;
        private String message;

        public LispFileNotFoundException(FileNotFoundException e) {
            this.message = e.getMessage();
        }

        @Override
        public int getSeverity() {
            return ErrorManager.CRITICAL_LEVEL;
        }

        @Override
        public String getMessage() {
            return message;
        }
    }

}
